# -*- coding: utf-8 -*-
from aiida.backends.utils import load_dbenv, is_dbenv_loaded
if not is_dbenv_loaded():
    load_dbenv()

import os
import sys
import time
from glob import glob
from os import path

from tailcorrection_workchain_blocked_pocket import TailCorrectionCheckBlockedPockets
from pathlib import Path

from aiida.orm import DataFactory
from aiida.orm.data.cif import CifData
from aiida.orm.data.base import Float
from aiida.orm.data.singlefile import SinglefileData
from aiida.work.run import run, submit
import random

NetworkParameters = DataFactory('zeopp.parameters')
ParameterData = DataFactory('parameter')

zeopp_options = {
    "resources": {
        "num_machines": 1,
        "tot_num_mpiprocs": 1,
    },
    "max_wallclock_seconds": 3 * 60 * 60,
    "withmpi": False,
}
zeopp_probe_radius = Float(
    1.86)  # It will create 8 pore blocks for test purpose
zeopp_atomic_radii_file = SinglefileData(file=os.path.abspath(
    "/home/kevin/Documents/uni/EPFL/master_thesis/cof_ml_work/featurization/zeopp.rad"
))  # Radius file for the framework


def multiply_unit_cell(cif, cutoff):
    """Returns the multiplication factors (tuple of 3 int) for the cell vectors
    that are needed to respect: min(perpendicular_width) > threshold
    """
    from math import cos, sin, sqrt, pi
    import numpy as np
    deg2rad = pi / 180.
    struct = cif.values.dictionary.itervalues().next()
    a = float(struct['_cell_length_a'])
    b = float(struct['_cell_length_b'])
    c = float(struct['_cell_length_c'])
    alpha = float(struct['_cell_angle_alpha']) * deg2rad
    beta = float(struct['_cell_angle_beta']) * deg2rad
    gamma = float(struct['_cell_angle_gamma']) * deg2rad
    v = sqrt(1 - cos(alpha)**2 - cos(beta)**2 - cos(gamma)**2 +
             2 * cos(alpha) * cos(beta) * cos(gamma))
    cell = np.zeros((3, 3))
    cell[0, :] = [a, 0, 0]
    cell[1, :] = [b * cos(gamma), b * sin(gamma), 0]
    cell[2, :] = [
        c * cos(beta),
        c * (cos(alpha) - cos(beta) * cos(gamma)) / (sin(gamma)),
        c * v / sin(gamma)
    ]
    cell = np.array(cell)
    diag = np.diag(cell)
    return tuple(int(i) for i in np.ceil(cutoff / diag * 2.))


forcefields = ['DREIDINGUFF-TraPPE_TC', 'DREIDINGUFF-TraPPE']
cutoffs = [
    6.0, 8.0, 10.0, 12.0, 14.0, 16.0, 18.0, 20.0, 22.0, 24.0, 26.0, 28.0, 30.0
]

structures = [
    '/home/kevin/Documents/uni/EPFL/master_thesis/tailcorrection/structures_for_study/TSC.cif'
]

for cif in structures:
    s = CifData(file=cif)
    s.label = Path(cif).stem
    for ff in forcefields:
        for cutoff in cutoffs:
            nx, ny, nz = multiply_unit_cell(s, cutoff)
            unitscells = "{} {} {}".format(nx, ny, nz)
            if cutoff == 20.0:
                raspa_params = ParameterData(
                    dict={
                        "GeneralSettings": {
                            "SimulationType": "MonteCarlo",
                            "CutOff": cutoff,
                            "ChargeMethod": "None",
                            "Forcefield": ff,
                            "UnitCells": unitscells,
                            "ExternalTemperature": 298.0,
                            "ComputeRDF": "yes",
                            "WriteRDFEvery": 1000,
                            'RemoveAtomNumberCodeFromLabel': 'yes',
                        },
                        "Component": [{
                            "MoleculeName": "methane",
                            "MoleculeDefinition": "TraPPE",
                        }],
                    })

            else:
                raspa_params = ParameterData(
                    dict={
                        "GeneralSettings": {
                            "SimulationType": "MonteCarlo",
                            "CutOff": cutoff,
                            "ChargeMethod": "None",
                            "Forcefield": ff,
                            "UnitCells": unitscells,
                            "ExternalTemperature": 298.0,
                            'RemoveAtomNumberCodeFromLabel': 'yes',
                        },
                        "Component": [{
                            "MoleculeName": "methane",
                            "MoleculeDefinition": "TraPPE",
                        }],
                    })

            outputs = submit(
                TailCorrectionCheckBlockedPockets,
                structure=s,
                zeopp_code=Code.get_from_string('zeopp@fidis'),
                _zeopp_options=zeopp_options,
                zeopp_probe_radius=zeopp_probe_radius,
                zeopp_atomic_radii=zeopp_atomic_radii_file,
                raspa_parameters=raspa_params,
                raspa_code=Code.get_from_string('raspa_modified@fidis'),
            )

            time.sleep(10)
